from Tkinter import *
import tkMessageBox
from tkFont import *
import sqlite3
import time

class Part(object):

    """
    this to avoid having to deal with typing quotes in a dictionary
    for just a part.  The dictionary key is the part number, and the
    part object has the description and the serialized part flag.
    
    """

    def __init__(self):
        description = ''
        serial_flag = ''
        machine_number = ''
        default_cmm = ''


class PartList(object):

    """
    This is a single use class for the list of
    part numbers, done to avoid having a bunch of
    methods inside the gui class.
    
    """

    def __init__(self, filename):
        self.parts = self.get_list(filename)


    def get_list(self, filename):
        
        """
        load a list of part numbers and descriptions,
        split them out into a dictionary with the part
        number as the dictionary key and the part info
        as object attributes.
        
        """

        file_name = open(filename, 'r')
        part_list = file_name.readlines()
        file_name.close()

        parts = {}

        for line in part_list:
            line = line.strip().split(',')
            parts[line[0]] = Part()
            parts[line[0]].description = line[1].strip()
            parts[line[0]].serial_flag = line[2].strip()
            parts[line[0]].machine_number = line[3].strip()
            parts[line[0]].default_cmm = line[4].strip()

        return parts

class MachineList(object):

    """
    This is a single use class for the list of
    machine numbers, done to avoid having a bunch of
    methods inside the gui class.
    
    """

    def __init__(self, filename):
        self.machine_list = self.load_machine_list(filename)

    def load_machine_list(self, filename):

        machine_list = {}
        
        filename = open(filename, 'rb')
        templist = filename.read().strip().split('\r\n')
        filename.close()

        for item in templist:
            item = item.split(', ')
            machine_list[item[0]] = item[1]

        return machine_list
    

class cmmWorkOrder(object):
    
    """ 
        This contains all the information
        for submitted work order to any CMM
    """
    
    def __init__(self):
    
        self.clock_number = ''
        self.part_number = ''
        self.quantity = ''
        self.serial_numbers = ''
        self.check_type = ''
        self.comments = ''
        self.raw_start_time = time.time() # this will be used for the work order number
        self.submit_time = time.ctime(self.raw_start_time) # human readable
        self.part_number = ''
        self.part_name = ''
        self.machine_number = ''
        self.shift = 1
        self.cmm_program = ''
        self.operator_name = ''

        # serial numbers is a list of serial numbers for a given submission, the work order number
        # in this case will be a combination of the work order number, plus the serial number, and
        # CMMLog will automatically generate a series of work orders for the submission.
        
        self.part_serial_numbers = []

        self.cmm_name = ''
        self.cmm_operator = ''
        self.cmm_start = '' # date cmm operator signed off on work order
        self.cmm_end = ''
        self.prog_start = ''
        self.prog_end = '' # for program run time only
        self.prog_run_time = ''
        self.total_time = '' # includes setup time, etc.
        self.new_prog_flag = '' # if a new program had to be written
        self.operator_comments = '' # cmm operator comments
        self.report_link = ''
        

class App(Frame):

    """
    This is the main gui.
    
    """

    # make menu entries at top of gui.
    
    def __init__(self, master):
        
        self.menubar = Menu(master)
        filemenu = Menu(self.menubar, tearoff=0)
        filemenu.add_command(label="New", command = self.donothing)
        filemenu.add_separator()
        filemenu.add_command(label="Exit", command = self.donothing)
        self.menubar.add_cascade(label="File", menu=filemenu)
        
        editmenu = Menu(self.menubar, tearoff=0)
        editmenu.add_command(label="Undo", command = self.donothing)
        editmenu.add_command(label="Clear Form", command = self.donothing)
        editmenu.add_separator()
        editmenu.add_command(label="Preferences", command = self.donothing)
        self.menubar.add_cascade(label="Edit", menu = editmenu)
        
        helpmenu = Menu(self.menubar, tearoff=0)
        helpmenu.add_command(label="Help", command = self.donothing)
        helpmenu.add_command(label="About...", command = self.donothing)
        self.menubar.add_cascade(label="Help", menu = helpmenu)
       
        master.config(menu=self.menubar)
        
        self.mainFont = Font(family = "Helvetica", size = 9, weight = 'bold')
        
        Frame.__init__(self, master)
        self.pack(padx = 5, pady = 5)
        self.makeWidgets()
        
    def donothing():
        pass
        
    def makeWidgets(self):
                
        # frame with clock number/entry
        # part number and entry
        # and quantity and entry
              
        # info frame
        
        self.infoFrame = Frame(self)
        self.infoFrame.grid(row = 1, column = 1, padx = 2)

        # STATUS LINE
           
        self.status_lab = LabelFrame(self.infoFrame, font = self.mainFont, text = 'STATUS :', relief = 'groove')
        self.status_bar = Label(self.status_lab, width = 80, text = "CREATE NEW CMM WORK ORDER", relief = 'sunken', anchor = 'w', justify = 'left', bg = 'yellow')
        self.status_lab.grid(row = 1, column = 1, columnspan = 10, padx = 2, pady = 2, sticky = 'w')
        self.status_bar.grid(row = 1, column = 2, padx = 2, pady = 2, sticky = 'w')

        self.shift_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = 'SHIFT', relief = 'groove')
        self.shift_var = StringVar(self.infoFrame)
        self.shift_var.set('1st')
        shift_list = ['1st', '2nd', '3rd']
        self.shift_choose = OptionMenu(self.shift_frm, self.shift_var, *shift_list)
        self.shift_choose.configure(width = 3)
        self.shift_choose.pack(padx = 2, pady = 2)
        self.shift_frm.grid(row = 2, column = 7, padx = 1, pady = 1, sticky = 'n')

        # CLOCK NUMBER, PART NUMBER, QUANTITY, PART NAME, MACHINE NUMBER
       
        self.clock_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "CLOCK #", relief = 'groove')
        self.pn_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "PART #", relief = 'groove')
        self.quantity_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "QTY", relief = 'groove')
        self.machine_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "MACHINE #", relief = 'groove')
        self.pname_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "PART NAME", relief = 'groove')
        self.cktype_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = "CHECK TYPE", relief = 'groove')
        
        self.clock_frm.grid(row = 2, column = 1, padx = 1, pady = 1, sticky = 'nw')
        self.pn_frm.grid(row = 2, column = 2, padx = 1, pady = 1, sticky = 'nw')
        self.quantity_frm.grid(row = 2, column = 3, padx = 1, pady = 1, sticky = 'nw')
        self.pname_frm.grid(row = 2, column = 4, padx = 1, pady = 1, sticky = 'nw')
        self.machine_frm.grid(row = 2, column = 5, padx = 1, pady = 1, sticky = 'nw')
        self.cktype_frm.grid(row = 2, column = 6, padx = 1, pady = 1, sticky = 'w')

        self.clock_entry = Entry(self.clock_frm, width = 6)
        self.machine_number = Label(self.machine_frm, width = 6)
        self.part_entry = Entry(self.pn_frm, width = 8)
        self.quantity_entry = Entry(self.quantity_frm, width = 3)
        self.part_name = Label(self.pname_frm, width = 20, relief = 'sunken')
        self.clock_entry.bind('<FocusOut>', self.validate_clocknum)
        self.clock_entry.pack(padx = 2, pady = 2)
        self.machine_number.pack(padx = 2, pady = 2)
        self.part_entry.bind('<FocusOut>', self.get_part_name)
        self.part_entry.pack(padx = 2, pady = 2)
        self.quantity_entry.bind('<FocusOut>', self.validate_quantity)
        self.quantity_entry.pack(padx = 2, pady = 2)
        self.part_name.pack(padx = 2, pady = 2)
                
        self.check_type_var = StringVar(self.cktype_frm)
        self.check_type_var.set('SPOT')
        check_types = ['SETUP', 'QUICK', 'SPOT', 'OTHER']
        self.check_type = OptionMenu(self.cktype_frm, self.check_type_var, *check_types)
        self.check_type.configure(width = 8)
        self.check_type.pack(padx = 2, pady = 2)

        self.prog_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = 'CHOOSE CMM PROGRAM')
        self.prog_var = StringVar(self.prog_frm)
        self.prog_var.set('NONE')
        v903_blocks = []
        v903_heads = []
        _30L_blocks = []
        prog_list = ['prog1', 'prog2 dfsdfsdfsdf', 'prog3']
        self.prog_choose = OptionMenu(self.prog_frm, self.prog_var, *prog_list)
        self.prog_choose.configure(width = 20, state = 'disabled')
        self.prog_choose.pack()
        self.prog_frm.grid(row = 4, column = 1, columnspan = 3)
      
        self.comment_frm = LabelFrame(self.infoFrame, font = self.mainFont, text = 'SPECIAL INSTRUCTIONS (IF ANY)')
        self.comment_text = Entry(self.comment_frm, width = 65, relief = 'sunken', bd = 2)
        self.comment_text.pack(padx = 2, pady = 2)
        self.comment_frm.grid(row = 4, column = 4, columnspan = 10, padx = 1, pady = 1, sticky = 'W')
         
        self.bottom_group = Frame(self.infoFrame)
        self.submit_button = Button(self.bottom_group, font = self.mainFont, text = "SUBMIT", bg = 'gray', width = 12, command = self.submit)
        self.cancel_button = Button(self.bottom_group, font = self.mainFont, text = "CLEAR FORM", bg = 'gray', width = 12, command = lambda: self.confirm_reset('YES'))
        self.submit_button.grid(row = 1, column = 2, padx = 2)
        self.cancel_button.grid(row = 1, column = 3, padx = 2)
        self.bottom_group.grid(row = 5, column = 1, columnspan = 10, pady = 2, sticky = 'e')

    def validate_serial(self, serial):
        print serial
        return serial


    def grab_serials(self, quantity):

        for i in range(1, quantity + 1):
            work_order.part_serial_numbers.append(self.validate_serial(self.s_ents[i].get()))
        print work_order.part_serial_numbers
        

    def input_serial(self, quantity):

        self.s_labs = {}
        self.s_ents = {}

        self.serials = []

        self.s_input = Toplevel()
        self.instructions = Label(self.s_input, text = 'Enter serial numbers in \"nnMyy\" format\n\
using the letters A-M for the month.\n\
Do NOT use the letter \'I\'', justify = 'left')
        
        self.instructions.pack()
        
        self.s_frm = LabelFrame(self.s_input, text = "ENTER SERIAL NUMBER(s)")
        self.s_frm.pack()

        for i in range(1, quantity + 1):
            self.s_labs[i] = Label(self.s_frm, text = 'SERIAL #' + str(i))
            self.s_ents[i] = Entry(self.s_frm)
            self.s_labs[i].grid(row = i, column = 1)
            self.s_ents[i].grid(row = i, column = 2)

        self.sn_bottom_frm = Frame(self.s_input)
        self.sn_bottom_frm.pack()

        self.sn_submit = Button(self.sn_bottom_frm, text = 'SUBMIT', command = lambda: self.grab_serials(quantity))
        self.sn_submit.grid(row = 1, column = 1)

    def validate_clocknum(self, dummy):

        clock_number = self.clock_entry.get()

        if not clock_number.isdigit():
            self.clock_entry.configure(bg = 'red', fg = 'white')
            self.status_bar.configure(text = 'Please correct areas highlighted in RED and resubmit')
        elif len(clock_number) < 5:
            self.clock_entry.configure(bg = 'red', fg = 'white')
            self.status_bar.configure(text = 'Please correct areas highlighted in RED and resubmit')
        else:
            self.clock_entry.configure(bg = 'white', fg = 'black')
            self.status_bar.configure(text = 'CREATE NEW CMM WORK ORDER')


    def get_part_name(self, dummy):

        """
        validates the part number against the list,
        if it is valid, gets the part name and
        machine number
        
        """

        parts = PartList('partlist2a.csv') 
        machine_list = MachineList('machines2.txt')
        
        part_number = self.part_entry.get()

        if part_number not in parts.parts.keys():
            self.part_entry.configure(bg = 'red', fg = 'white')
            self.status_bar.configure(text = 'Please correct areas highlighted in RED and resubmit')
        else:
            self.part_name.configure(text = '')
            self.part_name.configure(text = parts.parts[part_number].description)
            self.machine_number.configure(text = parts.parts[part_number].machine_number)
            self.status_bar.configure(text = 'CREATE NEW CMM WORK ORDER')
            if parts.parts[part_number].serial_flag == 'Y':
                self.part_entry.configure(bg = 'yellow', fg = 'black')
                self.status_bar.configure(text = 'Serialized Parts: Please input serial numbers')
            if parts.parts[part_number].default_cmm == 'DEA':
                print 'dea part'
       

    def validate_quantity(self, dummy):

        quantity = self.quantity_entry.get()

        try:
            test = int(quantity)
            self.quantity_entry.configure(bg = 'white', fg = 'black')
            self.status_bar.configure(text = 'CREATE NEW CMM WORK ORDER')
            if test > 20 and not tkMessageBox.askyesno(title = "QUANTITY HIGH", message = "QUANTITY SEEMS HIGH,\nIS IT CORRECT?"):
                self.quantity_entry.configure(bg = 'red', fg = 'white')
                self.status_bar.configure(text = 'Please correct areas highlighted in RED and resubmit')
        except:
            self.quantity_entry.configure(bg = 'red', fg = 'white')
            self.status_bar.configure(text = 'Please correct areas highlighted in RED and resubmit')
            error_found = 1

    def auto_fill(self):
        part_number = self.part_entry.get()      

    
    def submit(self):
        self.status_bar.configure(text = "VALIDATING INFORMATION...")
        work_order = cmmWorkOrder()
        work_order.clock_number = self.clock_entry.get()
        work_order.machine_number = self.machine_number.cget('text')
        work_order.part_number = self.part_entry.get()
        work_order.part_name = self.part_name.cget('text')
        work_order.quantity = self.quantity_entry.get()
        work_order.check_type = self.check_type_var.get()
        
        work_order.shift = self.shift_var.get()
        work_order.cmm_program = ''
        am = self.prog_var.get()
        work_order.comments = self.comment_text.get()

        if tkMessageBox.askyesno(title = "CONFIRM SUBMIT", message = "Confirm Submission?"):
            self.write_to_queue(work_order)
            self.confirm_reset('NO')
 
    def reset_form(self):
        
        self.clock_entry.focus_set()
        self.clock_entry.delete(0, END)
        self.clock_entry.configure(bg = 'white', fg = 'black')
        self.machine_number.configure(text = '')
        self.part_entry.delete(0, END)
        self.part_entry.configure(bg = 'white', fg = 'black')
        self.quantity_entry.delete(0, END)
        self.quantity_entry.configure(bg = 'white', fg = 'black')
        self.part_name.configure(text = '')
        self.comment_text.delete(0, END)
        self.status_bar.configure(text = "CREATE NEW CMM WORK ORDER")
        self.check_type_var.set('SPOT')
        self.shift_var.set('1st')
        
    def confirm_reset(self, askflag):

        if askflag == 'YES':
            if tkMessageBox.askyesno(title = "CANCEL", message = "Clear Form?"):
                self.reset_form()
        else:
            self.reset_form()

    def write_to_queue(self, work_order):

        print work_order.raw_start_time
        print work_order.clock_number
        print work_order.part_number
        print work_order.part_name
        print work_order.quantity
        print work_order.check_type
        print work_order.comments


root = Tk()
root.title("CMMLog 2.0a")
app = App(root)
root.mainloop()
